/* funcs.h  */
/* header file for funcs.c */
/* 92/11/17 new made by I.Yamamura */
/* funcs.c itself does not call this header file */
/* 93/01/09 extern declinationss */

/* ============================= contents ================================ */
/* ----- allocate and free 2-dimensional array, double x[m][n]             */
/*    double **d_alloc(m, n)                                               */
/*    void d_free(x, m)                                                    */
/*       int m, n;                                                         */
/*       double **x;                                                       */
/* ----- allocate and free 3-dimensional array, double x[l][m][n]          */
/*    double ***t_alloc(l, m, n)                                           */
/*    void t_free(x, l, m)                                                 */
/*       int l, m, n;                                                      */
/*       double ***x;                                                      */
/* ----- Pre process for the making spline table                           */
/*    void SplinePreProcess(n, x, y)                                       */
/*       int n;                                                            */
/*       double *x, *y;                                                    */
/* ----- make spline table                                                 */
/*    void make_sp_table(n, x, y, z)                                       */
/*       int n;                                                            */
/*       double *x, *y, *z;                                                */
/* ----- spline interpolation                                              */
/*    double spline(t, n, x, y, z)                                         */
/*       int n;                  /* number of data                         */
/*       double t, *x, *y, *z;   /* t: input x, x,y: data, z: work area    */
/* ----- 2 Dimensional spline interpolation                                */
/*    void make_sp_table2D(m, n, x, y, z, w)                               */
/*    double spline2D(tx, ty, m, n, x, y, z, w, w2, w3)                    */
/*       double tx, ty; /* position to be interpolated                     */
/*       int m, n;      /* dimension of x and y                            */
/*       double *x;     /* x[m]: position of x                             */
/*       double *y;     /* y[n]: position of y                             */
/*       double **z;    /* z[m][n]: value at (x, y)                        */
/*       double **w;    /* working area w[m][n]                            */
/*       double *w2;    /* working area w2[m]                              */
/*       double *w3;    /* working area w3[m]                              */
/* ----- 2 Dimensional spline interpolation                                */
/*    void make_sp_table3D(l, m, n, x, y, z, v, w)                         */
/*    double spline3D(tx, ty, l, m, n, x, y, z, v, w, w2, w3, w4, w5)      */
/*       double tx, ty, tz; /* position to be interpolated                 */
/*       int l, m, n;   /* dimension of x and y                            */
/*       double *x;     /* x[l]: position of x                             */
/*       double *y;     /* y[m]: position of y                             */
/*       double *z;     /* z[n]: position of y                             */
/*       double ***v;   /* v[l][m][n]: value at (x, y, z)                  */
/*       double ***w;   /* working area w[l][m][n]                         */
/*       double *w2;    /* working area w[m]                               */
/*       double *w3;    /* working area w[m]                               */
/*       double *w4;    /* working area w[l]                               */
/*       double *w5;    /* working area w[l]                               */
/* ----- interploate by linear line. 93/02/23                              */
/*    double LinerInterpolate(x, n, xx, yy)                                */
/*       double x;                                                         */
/*   int n;      /* size of xx and yy, n < 0 means the inverse order of xx */
/*   double *xx, *yy;                                                      */
/* ----- convert string to double.                                         */
/*    double atod(s)                                                       */
/*       char *s;                                                          */
/* ----- reast square methord                                              */
/* ----- fit data to y = a[0] + a[1]*x + a[2]*x^2 + ... + a[n-1]*x^n-1     */
/*    void rsm(n, x, y, m, a)                                              */
/*       int n;         /* number of data                                  */
/*       double *x, *y; /* data to be fit                                  */
/*       int m;         /* dimension                                       */
/*       double *a;     /* m-th array of result                            */
/* ----- Solve arbitrary function by Newton-Raphson methord.               */
/*       number of loops necessary to converge is returned.                */
/*    int Newton(Func, x, lim, max)                                        */
/*    int LogNewton(Func, x, lim, max)                                     */
/*       double (*Func)();    /* function to be solved                     */ 
/*       double *x;           /* initial value (also returned value)       */
/*       double lim;          /* convergence limit                         */
/*       int max;             /* maximum limit of iteration                */
/* ----- find root of a function Func() by fallse posiition methord.       */
/*    int flspos(Func, x, a, b, tolerance, imax)                           */
/*       double *x;           /* return value                              */
/*       double a, b;         /* find solution between a and b             */
/*       double tolerance;                                                 */
/*       double (*Func)();    /* function to be solved                     */
/*       int imax;            /* repeat befor useing bisection methord.    */
/* ----- rotate the cordinate                                              */
/* (X, Y) is the position on the corrdinate which is rotated by an angle   */
/* theta to counter-clockwize from the coordinate which (x, y) is on       */
/*    void Rotate(x, y, X, Y, theta)                                       */
/*       double x, y;         /* position befor rotation                   */
/*       double *X, *Y;       /* position after rotation                   */
/*       double theta;        /* angle to rotate                           */
/* ----- cartician corrdinate to spherical coordinate                      */
/*    void Car2Sph(x, y, z, r, theta, phai)                                */
/*       double x, y, z;            /* cartician coordinate                */
/*       double *r, *theta, *phai;  /* spherical coordinate                */
/*                                                                         */
/* ----- translate all characters in the string upper/lower case           */
/*    char *strupper(s)                                                    */
/*    char *strlower(s)                                                    */
/*       char *s;                                                          */
/* ----- convert degree to 60-th style                                     */
/*    void deg60(d, dd, md, sd)                                            */
/*       double d;               /* degree                                 */
/*       double *dd, *md, *sd;   /* returned values                        */
/*                                                                         */
/* ======================================================================= */

/* ============================== constants ============================== */
#ifndef DBL_EPSILON
#  define DBL_EPSILON 2.2204460492503131e-16 /* same as of MIPS and HP */
#endif
#define EPSILON DBL_EPSILON
/* ============================== declinations =========================== */
extern double **d_alloc();
extern void d_free();
extern double ***t_alloc();
extern void t_free();
extern void SplinePreProcess();
extern void make_sp_table();
extern double spline();
extern void make_sp_table2D();
extern double spline2D();
extern void make_sp_table3D();
extern double spline3D();
extern double LinerInterpolate();
extern double atod();
extern void rsm();
extern int Newton();
extern int LogNewton();
extern int flspos();
extern void Rotate();
extern void Car2Sph();
extern char *strupper();
extern char *strlower();
extern void deg60();

/* end of funcs.h */
