/* matrix.c */
/* subroutines matrix calculation */
/* originary programed by I.Yamamura 91/11 for ATR */
/* reassembled by I.Yamamura 91/12/08 */
/* bug fix on solve() 93/09/28 */

#include <stdio.h>
#include <math.h>
#include <malloc.h>  /* 92/08/12 */

#include "matrix.h"

#define sqr(x) ((x)*(x))
#define sp(a) puts("a")    /* for debugging */

/* --------------------------------------------------- */
void mul_matrix(n, a, c, r)
/* multiple matrix a*c=r */
   int n;
   double **a, **c, **r;
{
   int i, j, k;

   for (i = 0; i < n; i++) {
      for (j = 0; j < n; j++) {
         r[i][j] = 0.0;
         for (k = 0; k < n; k++)
            r[i][j] = r[i][j] + a[i][k]*c[k][j];
      }
   }
} /* mul_matrix */

/* --------------------------------------------------- */
int inverse(n, a, v)
/* make inverse matrix a(-1) = v */
   double **a, **v;
   int n;
{
   int i, j, *ip, er;
   double *b, *x;

   b = (double *)calloc((unsigned)n, (unsigned)sizeof(double));
   x = (double *)calloc((unsigned)n, (unsigned)sizeof(double));
   ip = (int *)calloc((unsigned)n, (unsigned)sizeof(int));

   er = decomp(n, a, ip);
   for (i = 0; i < n; i++) {
      for (j = 0; j < n; j++) {
         if (j == i)
            b[j] = 1;
         else
            b[j] = 0;
      }
      solve(n, a, b, x, ip);
      for (j = 0; j < n; j++)
         v[j][i] = x[j];
   }

   free((char *)b);
   free((char *)x);
   free((char *)ip);

   return er;

} /* inverse */

/* --------------------------------------------------- */
void renormalize(n, e, r)
/* renormalize vector e(ij) by Gram-Schmidt formula */
   double **e, **r;
   int n;
{
   int i, j, k;
   double *pr, sumnorm, norm;

   pr = (double *)calloc((unsigned)n, (unsigned)sizeof(double));

   for (i = 0; i < n; i++) {
      for (k = 0; k < i; k++)
         pr[k] = in_product(n, e[i], e[k]);

      sumnorm = 0.0;
      for (j = 0; j < n; j++) {
         r[i][j] = e[i][j];
         for (k = 0; k < i; k++)
            r[i][j] = r[i][j] - pr[k] * e[k][j];
         sumnorm = sumnorm + sqr(r[i][j]);
      }
      norm = sqrt(sumnorm);
      for (j = 0; j < n; j++)
         r[i][j] = r[i][j] / norm;
   }
   free((char *)pr);

} /* renormalize */

/* --------------------------------------------------- */
void copy_matrix(n, a, b)
   double **a, **b;
   int n;
{
   int i, j;

   for (i = 0; i < n; i++)
      for (j = 0; j < n; j++)
         b[i][j] = a[i][j];
} /* copy_matrix */

/* --------------------------------------------------- */
void show_matrix(n, a)
   double **a;
   int n;
{
   int i, j;

   printf("\n");
   for (i = 0; i < n; i++) {
      for (j = 0; j < n; j++)
         printf("%15e ",a[i][j]);
      printf("\n");
   }
} /* show_matrix */

/* --------------------------------------------------- */
void show_vector(n, v)
   double *v;
   int n;
{
   int i;

   printf("\n");
   for (i = 0; i < n; i++)
      printf("%15e ", v[i]);
   printf("\n");

} /* show_vector */

/* --------------------------------------------------- */
double in_product(n, a, b)
/* calculate inner products of vector a and b */
   double *a, *b;
   int n;
{
   int i;
   double r = 0.0;

   for (i = 0; i < n; i++)
      r =  r + a[i] * b[i];

   return r;
} /* in_product */

/* --------------------------------------------------- */
int decomp(n, a, ip)
/* DU decomposition matrix a */
   double **a;
   int *ip;
{
#define EPS 1.0E-75
   int i, j, k, l;
   double al, lv;

   for (k = 0; k < n; k++)
      ip[k] = k;

   for (k = 0; k < n; k++) {
      l = k;
      al = fabs(a[ip[l]][k]);
      for (i = k + 1; i < n; i++) {
         if (fabs(a[ip[i]][k]) > al) {
            l = i;
            al = fabs(a[ip[l]][k]);
         }
      }
      if (l != k) {
         lv = ip[k];
         ip[k] = ip[l];
         ip[l] = lv;
      }
      if (al <= EPS) {
/*         printf("(DECOMP) matrix singular (at %4d-th pivot)", k); */
         n = -k;
         return -1;
      }

      a[ip[k]][k] = 1.0 / a[ip[k]][k];

      for (i = k + 1; i < n; i++) {
         a[ip[i]][k] = a[ip[i]][k] * a[ip[k]][k];
         for (j = k + 1; j < n; j++) {
            a[ip[i]][j] = a[ip[i]][j] - a[ip[i]][k] * a[ip[k]][j];
         }
      }
   }

   return 0;
} /* decomp */

/* --------------------------------------------------- */
double EstCond(n, a)
/* estimate condition number for mattrix a */
   int n;
   double **a;
{
   int i, k;
   double *v, *y;
   double s, t, anorm, ymax;
   int *ip;

   v = (double *)calloc((unsigned)n, (unsigned)sizeof(double));
   y = (double *)calloc((unsigned)n, (unsigned)sizeof(double));
   ip = (int *)calloc((unsigned)n, (unsigned)sizeof(int));

   anorm  = 0.0;
   for (k = 0; k < n; k++) {
      s = 0.0;
      for (i = 0; i < n; i++)
         s += fabs(a[i][k]);
      if (s > anorm) anorm = s;
   }

   decomp(n, a, ip);

   for (k = 0; k < n; k++) {
      t = 0.0;
      for (i = 0; i < k - 1; i++)
         t -= a[ip[i]][k] * (double)v[i];

      s = 1.0;
      if (t < 0.0) s = -1.0;
      v[k] = (s + t) * a[ip[k]][k];
   }
   for (k = n - 1; k >= 0; k--) {
      t = v[k];
      for (i = k + 1; i < n; i++)
         t -= a[ip[i]][k] * (double)y[ip[i]];
      y[ip[k]] = t;
   }

   ymax = 0.0;
   for (i = 0; i < n; i++)
      if (fabs(y[i]) > ymax) ymax = fabs(y[i]);

   free((char *)v);
   free((char *)y);
   free((char *)ip);

   return anorm * ymax;
} /* EstCond() */

/* --------------------------------------------------- */
void solve(n, a, b, x, ip)
/* solve a*b=x */
/* 93/09/28 bug fix on last for() statement (n - 1 was n) */
   double **a, *b, *x;
   int n, *ip;
{
   int i, j;
   double t;

   for (i = 0; i < n; i++) {
      t = b[ip[i]];
      for (j = 0; j <= i - 1; j++) {
         t = t - a[ip[i]][j] * x[j];
      }
      x[i] = t;
   }
   for (i = n - 1; i >= 0; i--) {
      t = x[i];
      for (j = i + 1; j <= n - 1; j++) {
         t = t - a[ip[i]][j] * x[j];
      }
      x[i] = t * a[ip[i]][i];
   }
} /* solve */

/* --------------------------------------------------- */
void yLins(n, a, b, x)
/* just call decomp() and solve() simulteneously */
   int n;
   double **a, *b, *x;
{
   int *ip;

   ip = (int *)calloc((unsigned)n, (unsigned)sizeof(int));
   decomp(n, a, ip);
   solve(n, a, b, x, ip);
   free((char *)ip);

} /* yLins */

/* end of matrix.c */
