/* output.c */
/* write result in fits format */
/* 93/08/28 I.Yamamura */
/* 03/01/02 set ORIGIN properly to avoid crash */

#include <stdio.h>
#include <math.h>
#include <malloc.h>
#include <string.h>

#include "const.h"         /* physical constants */
#include "macro.h"         /* common macro functions */
#include "funcs.h"

#include "fits.h"
#include "rt.h"

void WriteOneSp(ofn, spc, sf, ctype, unit)
/* write in spectral fits format */
/* 93/08/28 I.Yamamura */
   char *ofn;        /* output file name */
   double *spc;      /* data */
   AXIS  *sf;        /* frequency data */
   char *ctype;      /* axis type */
   int   unit;       /* unit of data */
{
   fits_header f;
   long  *l;
   int i;

   f.simple    = 'T';
   f.bitpix    = 32;
   f.naxis     = 2;
   f.naxis1    = 0;
   f.naxis2    = sf->n;
   f.groups    = 'T';
   f.pcount    = 22;
   f.gcount    = 1;     /* only one spectrum is written */

   strcpy(f.object,   "CALCULAT");
   strcpy(f.origin,   "ORIGIN");
   strcpy(f.instrume, "RT      ");
   strcpy(f.telescop, "MIPS-WS ");
   strcpy(f.observer, "YAMAMURA");
   f.epoch     = 1950.0;
   
   f.p = (p_struct *)calloc(f.pcount, sizeof(p_struct));
   strcpy(f.p[ 0].type, "CRVAL2  ");   f.p[ 0].scal  =     1.000000;
   strcpy(f.p[ 1].type, "CDELT2  ");   f.p[ 1].scal  =     0.001000;
   strcpy(f.p[ 2].type, "RA      ");   f.p[ 2].scal  = 0.100000E-04;
   strcpy(f.p[ 3].type, "DEC     ");   f.p[ 3].scal  = 0.100000E-04;
   strcpy(f.p[ 4].type, "RAOJ    ");   f.p[ 4].scal  = 0.100000E-04;
   strcpy(f.p[ 5].type, "DECOJ   ");   f.p[ 5].scal  = 0.100000E-04;
   strcpy(f.p[ 6].type, "DRA     ");   f.p[ 6].scal  = 0.100000E-04;
   strcpy(f.p[ 7].type, "DDEC    ");   f.p[ 7].scal  = 0.100000E-04;
   strcpy(f.p[ 8].type, "RAF     ");   f.p[ 8].scal  = 0.100000E-04;
   strcpy(f.p[ 9].type, "DECF    ");   f.p[ 9].scal  = 0.100000E-04;
   strcpy(f.p[10].type, "DRAF    ");   f.p[10].scal  = 0.100000E-04;
   strcpy(f.p[11].type, "DDECF   ");   f.p[11].scal  = 0.100000E-04;
   strcpy(f.p[12].type, "FREQ    ");   f.p[12].scal  =  1000.000000;
   strcpy(f.p[13].type, "FQTRK   ");   f.p[13].scal  =  1000.000000;
   strcpy(f.p[14].type, "FQIF1   ");   f.p[14].scal  =  1000.000000;
   strcpy(f.p[15].type, "DATE-OBS");   f.p[15].scal  =     0.001000;
   strcpy(f.p[16].type, "ARRAY-NO");   f.p[16].scal  =     1.000000;
   strcpy(f.p[17].type, "IAVTM   ");   f.p[17].scal  =     1.000000;
   strcpy(f.p[18].type, "NAVSP   ");   f.p[18].scal  =     1.000000;
   strcpy(f.p[19].type, "RMS     ");   f.p[19].scal  =     0.000010;
   strcpy(f.p[20].type, "NBASE   ");   f.p[20].scal  =     0.000010;
   strcpy(f.p[21].type, "SCFACT  ");   f.p[21].scal  =     0.000010;

   strcpy(f.ctype2, ctype);
   f.crpix2    = sf->rpix + 1;
   f.crval2    = sf->rval;
   f.cdelt2    = sf->delt;
   f.bscale    = 0.0001;
   f.bzero     = 0.0;
   f.size      = sf->n + f.pcount;

        if (unit == TA) strcpy(f.bunit, "K       ");
   else if (unit == JY) strcpy(f.bunit, "JY      ");
   else                 strcpy(f.bunit, "        ");

   l = (long *)calloc(f.size, sizeof(long));

   l[ 0] = (long)(sf->rval / f.p[ 0].scal);
   l[ 1] = (long)(sf->delt / f.p[ 1].scal);
   l[ 2] = (long)(0.0      / f.p[ 2].scal);
   l[ 3] = (long)(0.0      / f.p[ 3].scal);
   l[ 4] = (long)(0.0      / f.p[ 4].scal);
   l[ 5] = (long)(0.0      / f.p[ 5].scal);
   l[ 6] = (long)(0.0      / f.p[ 6].scal);
   l[ 7] = (long)(0.0      / f.p[ 7].scal);
   l[ 8] = (long)(0.0      / f.p[ 8].scal);
   l[ 9] = (long)(0.0      / f.p[ 9].scal);
   l[10] = (long)(0.0      / f.p[10].scal);
   l[11] = (long)(0.0      / f.p[11].scal);
   l[12] = (long)(sf->rval / f.p[12].scal);
   l[13] = (long)(sf->rval / f.p[13].scal);
   l[14] = (long)(1.0      / f.p[14].scal);
   l[15] = (long)(0.0      / f.p[15].scal);
   l[16] = (long)(1.0      / f.p[16].scal);
   l[17] = (long)(0.0      / f.p[17].scal);
   l[18] = (long)(0.0      / f.p[18].scal);
   l[19] = (long)(0.0      / f.p[19].scal);
   l[20] = (long)(1.0      / f.p[20].scal);
   l[21] = (long)(1.0      / f.p[21].scal);

   if (strncmp(ctype, "VELO", 4) == 0) {
      for (i = 0; i < sf->n; i++)
         l[(sf->n - 1 - i) + f.pcount] = (long)((spc[i] - f.bzero) / f.bscale);
   } else {
      for (i = 0; i < sf->n; i++)
         l[i + f.pcount] = (long)((spc[i] - f.bzero) / f.bscale);
   }
   WriteSpectra(ofn, l, &f, 1);

} /* WriteOneSp() */

void WriteMap(ofn, dimg, d1, d2, d3, ctype3, unit)
/* write in image fits format */
/* 93/08/28 I.Yamamura */
   char *ofn;              /* output file name */
   double ***dimg;         /* 3-Dimensional image data */
   AXIS  *d1, *d2, *d3;    /* dimension data */
   char *ctype3;           /* type of 3rd axis */
   int   unit;             /* unit of data */
{
   fits_header f;
   long  *l;
   int i, i1, i2, i3;
   double dmax, dmin;

   f.simple    = 'T';
   f.bitpix    = 32;
   f.naxis     = 3;
   f.naxis1    = d1->n;    if (f.naxis1 == 0) f.naxis1 = 1;
   f.naxis2    = d2->n;    if (f.naxis2 == 0) f.naxis2 = 1;
   f.naxis3    = d3->n;    if (f.naxis3 == 0) f.naxis3 = 1;
   f.extend    = 'T';
   f.blocked   = 'T';

   strcpy(f.object,   "CALCULAT");
   strcpy(f.instrume, "RT      ");
   strcpy(f.telescop, "MIPS-WS ");
   strcpy(f.observer, "YAMAMURA");
        if (unit == TA) strcpy(f.bunit, "K /BEAM ");
   else if (unit == JY) strcpy(f.bunit, "JY/BEAM ");
   else                 strcpy(f.bunit, "        ");

   f.epoch     = 1950.0;
   f.obsra     = 0.0;
   f.obsdec    = 0.0;
   
   strcpy(f.ctype1, "RA---SIN");
   f.crval1    = d1->rval / 3600.0;
   f.cdelt1    = d1->delt / 3600.0;
   f.crpix1    = d1->rpix + 1;

   strcpy(f.ctype2, "DEC--SIN");
   f.crval2    = d2->rval / 3600.0;
   f.cdelt2    = d2->delt / 3600.0;
   f.crpix2    = d2->rpix + 1;

   if (strncmp(ctype3, "VELO", 4) == 0) {
      strcpy(f.ctype3, ctype3);
      f.crval3    = d3->rval;
      f.cdelt3    = d3->delt;
      f.crpix3    = d3->rpix + 1;
   } else if (strncmp(ctype3, "FREQ", 4) == 0) {
      strcpy(f.ctype3, ctype3);
      f.crval3    = d3->rval;
      f.cdelt3    = d3->delt;
      f.crpix3    = d3->rpix + 1;
   } else
      fprintf(stderr, "WriteMap : Unknow ctype3\n");

   f.size      = f.naxis1 * f.naxis2 * f.naxis3;

   dmax = dmin = dimg[0][0][0];
   for (i3 = 0; i3 < f.naxis3; i3++)
      for (i2 = 0; i2 < f.naxis2; i2++)
         for (i1 = 0; i1 < f.naxis1; i1++) {
            if (dimg[i3][i2][i1] > dmax) dmax = dimg[i3][i2][i1];
            if (dimg[i3][i2][i1] < dmin) dmin = dimg[i3][i2][i1];
         }

   f.datamax = dmax;
   f.datamin = dmin;
   f.bscale = (dmax - dmin) / pow(2.0, (double)f.bitpix);
   if (f.bscale < 1.0e-10) f.bscale = 1.0e-10;
   f.bzero = (dmax + dmin) / 2.0;
/* MAXLONG is nearly equal to 2.0e9 */
   while (fabs(f.bzero/f.bscale) > 2.0e9) f.bscale *= 10.0;

   l = (long *)calloc(f.size, sizeof(long));
   i = 0;
   if (strncmp(ctype3, "VELO", 4) == 0) {
      for (i3 = f.naxis3 - 1; i3 >= 0; i3--)
         for (i2 = 0; i2 < f.naxis2; i2++)
            for (i1 = 0; i1 < f.naxis1; i1++) {
               l[i++] = (dimg[i3][i2][i1] - f.bzero) / f.bscale;
            }
   } else {
      for (i3 = 0; i3 < f.naxis3; i3++)
         for (i2 = 0; i2 < f.naxis2; i2++)
            for (i1 = 0; i1 < f.naxis1; i1++) {
               l[i++] = (dimg[i3][i2][i1] - f.bzero) / f.bscale;
            }
   }
   WriteImage(ofn, l, &f, 1);

} /* WriteMap() */

/* end of output.c */