/* rate.c */
/* determine energy population by solving rate equation */
/* 92/11/16 almost completely re-writed */
/*          temporary, vibrational transittion cannot be treated */
/* 93/03/04 Rate() returns chk */

#include <stdio.h>
#include <math.h>
#include <malloc.h>

#include "const.h"         /* physical constants */
#include "macro.h"         /* common macro functions */
#include "funcs.h"
#include "matrix.h"

#include "mol.h"
#include "model.h"
#include "rt.h"

#define lrg(i, k) ((i) >= (k) ? (i) : (k))
#define sml(i, k) ((i) <= (k) ? (i) : (k))

/* ============================ declarations ============================= */
double RateEq();
double Tau2Beta();
double I_HII();
void CalcBeta();
void CalcBetaV();

/* ============================= main routine ============================ */
double RateEq(x, y, z, tk, v, dvdr, mol)
   double x, y, z;   /* position */
   double tk;        /* kinetic temperature */
   double v;         /* radial velocity */
   double dvdr;      /* dv /dr */
   MOLECULE *mol;    /* molecular data */
{
#define MAXLOOP 10
#define CHKLIM 1.0E-2

   int loop = 0;

   int i, k;
   int v1, j1, v2, j2;
   double r;
   double beta, ibeta;
   double **p, *q, *t;
   double chk, ltex;

   r = sqrt(sqr(x) + sqr(y) + sqr(z));

   p = d_alloc(mol->LV+1, mol->LV+1);
   q = (double *)calloc((unsigned)(mol->LV+1), (unsigned)sizeof(double));
   t = (double *)calloc((unsigned)(mol->LV+1), (unsigned)sizeof(double));

/* ----- Loop begins here ----- */
   loop = 0;
   do {

/* ----- Clear matrix ----- */
      for (i = 0; i < mol->LV; i++) {
         for (k = 0; k < mol->LV; k++)
            p[i][k] = 0.0;
         q[i] = 0.0;
      }

/* --- Make Matrix elements */
      for (v1 = 0; v1 < mol->NV; v1++) for (j1 = 0; j1 < mol->NJ; j1++) {
         i = v1 * mol->NJ + j1;
         for (v2 = 0; v2 < mol->NV; v2++) for (j2 = 0; j2 < mol->NJ; j2++) {
            k = v2 * mol->NJ + j2;
            if (v1 == v2) {
               if (j2 == j1 - 1) {
                  CalcBeta(r, x, y, z, tk, v, dvdr, i, mol, &beta, &ibeta);
                  p[i][i] -= mol->a[i][k] * mol->g[i] * (beta + ibeta);
                  p[i][k] += mol->a[i][k] * mol->g[i] * ibeta;
               } else if (j2 == j1 + 1) {
                  CalcBeta(r, x, y, z, tk, v, dvdr, k, mol, &beta, &ibeta);
                  p[i][i] -= mol->a[k][i] * mol->g[k] * ibeta;
                  p[i][k] += mol->a[k][i] * mol->g[k] * (beta + ibeta);
               }
            } else if (v2 == v1 + 1) {
               CalcBetaV(r, x, y, z, tk, v, dvdr, k, i, mol, &beta, &ibeta);
               p[i][i] -= mol->a[k][i] * mol->g[k] * ibeta;
               p[i][k] += mol->a[k][i] * mol->g[k] * (beta + ibeta);
            } else if (v2 == v1 - 1) {
               CalcBetaV(r, x, y, z, tk, v, dvdr, i, k, mol, &beta, &ibeta);
               p[i][i] -= mol->a[i][k] * mol->g[i] * (beta + ibeta);
               p[i][k] += mol->a[i][k] * mol->g[i] * ibeta;
            }
            p[i][i] -= mol->ct[i][k] * mol->g[i];
            p[i][k] += mol->ct[k][i] * mol->g[k];
         }
      }

      for (i = 0; i < mol->LV; i++) {
         p[mol->LV][i] = (double)mol->g[i];
         q[i] = 0.0;
      }
      q[mol->LV] = mol->nmol;

/* show_vector(mol->LV+1, mol->ng); /**/
/* show_matrix(mol->LV+1, p);  /**/
/* show_vector(mol->LV+1, q);  /**/

      yLins(mol->LV+1, p, q, t);

/* In order to avoid unreasonable value off populations, too small or */
/* negative value is replaced with previous value times LTE ratio.    */
      if (t[0] < 1.0e-64) t[0] = mol->ng0[0];
      for (i = 1, ltex = tk; i < mol->LV; i++) {
         if (t[i] < 1.0e-64) {
            ltex = Tex((*mol), i, i-1);
            t[i] = t[i-1] * mol->g[i] / (double)mol->g[i-1] *
                  exp(-mol->de[i][i-1]/WK/ltex);
         }
      }

/* ----- check if solutions are converged? ----- */
      for (i = 0, chk = 0; i < mol->LV; i++) {
         if ((fabs(mol->ng[i]) > 1.0e-20) && (fabs(t[i]) > 1.0e-20))
            chk += sqr((mol->ng[i] - t[i])/ mol->ng[i]);
         mol->ng[i] = t[i];
      }

      chk = sqrt(chk) / (double)(mol->LV+1);
/* ----- end of loop ----- */
   } while ((loop++ < MAXLOOP) && (chk > CHKLIM));

/*
   if ((loop > MAXLOOP) && (chk > CHKLIM))
      fprintf(stderr, "Over count!!! %8.3e %8.3f\n", r, chk);
*/

   d_free(p, mol->LV+1);
   free((char *)q);
   free((char *)t);

   return chk;
} /* RateEq() */

/* ================= subroutines for RateEq ================== */
/* ------------------------------------------------------------- */
double Tau2Beta(tau)
   double tau;
{
   if (tau < -50.0)
      return 1.0e20;    /* = (1.0 - exp(50.0)) / (-50.0) */
   else if (tau > 50.0)
      return 1.0 / tau;
   else if (fabs(tau) < 1.0e-10)
      return 1.0;
   else
      return (1.0 - exp(-tau)) / tau;

} /* Tau2Beta() */

/* ------------------------------------------------------------- */
double I_HII(r, X, Y, Z, theta, phai, nyu)
/* return emergent intensity from HII core seeing from (x, y, z) */
/* and angle theta and phai from line of the center */
/* 93/08/22 */
   double r, X, Y, Z;      /* position */
   double theta;           /* angle from the line of sight */
   double phai;            /* angle around the line of sight */
   double nyu;             /* freqency */
{
   double x, y, z;
   double alpha;           /* angle of the line of sight and x-axis */
   double cs, si;
   double ta;
   double dx;
   double te, ne;
   double tau, et;
   double tb;
   double hro;

   alpha = acos(X / r);
   ta = tan(theta - alpha);
   cs = cos(phai);
   si = sin(phai);
   hro = RO_HII();

   tb = BB(T0, nyu);
   dx = hro / 10.0;
   for (x = -hro; x <= hro; x += dx) {
      y = (x - X) * ta * cs;
      z = (x - X) * ta * si;
      if (Where(x, y, z) == 1) {
         ContEnv(x, y, z, &te, &ne);
         if (te == 0.0) tau = 0.0;
         else tau = 9.77e-3 * sqr(ne) / (te * sqrt(te)) / sqr(nyu)
            * (17.7 + log(te * sqrt(te) / nyu)) * dx;
         et = exp(-tau);
         tb = tb * tau + BB(te, nyu) * (1.0 - et);
      }
   }
   return tb;
} /* I_HII() */

/* ------------------------------------------------------------- */
void CalcBeta(r, x, y, z, tk, v, dvdr, JU, mol, beta, ibeta)
/* 92/11/16 Totaly rewrited. */
/*          global variables are not used. */
/* 93/08/21 modified. */
/*          model parameters are reffered trough functions */
/*          assuming spherically symmetric HII core */
   double r, x, y, z;      /* position */
   double tk;              /* kinetic temperature */
   double v;               /* radial velocity */
   double dvdr;            /* dv / dr */
   int JU;                 /* Upper state */
   MOLECULE *mol;          /* molecular data */
   double *beta, *ibeta;   /* escape probabilities */
{
#define THDIV 10     /* number of step of theta */
#define SA(theta) (1.0-cos(theta)) /* solid angle / 4*PI*2 */
   
   int JL = JU - 1;

   double theta, theta_s, theta_h, dth;
   double tau0, tau, bt, sa;
   double ih2;

   double sumds = 0;
   double sumbeta = 0;
   double sumibeta = 0;

   tau0 = cub(C / mol->nyu0[JU][JL]) * mol->g[JU] * mol->a[JU][JL]
         * (mol->ng[JL] - mol->ng[JU]) / (8.0 * PI) * r / v;

/* ----- from central star ----- */
   theta_s = atan(R_Star() / r);
   tau = tau0 / (1.0 + sqr(cos(theta_s/2.0)) * (dvdr - 1.0));

   bt = Tau2Beta(tau);
   sa = SA(theta_s);
   sumds    += sa;
   sumbeta  += sa * bt;
   sumibeta += sa * bt * BB(T_Star(), mol->nyu0[JU][JL]);

/* ----- from HII region ----- */
   if ((theta_h = atan(RO_HII())) > theta_s) {
      dth = (theta_h - theta_s) / (double)THDIV;
      for (theta = theta_s; theta < theta; theta += dth) {

         tau = tau0 / (1.0 + sqr(cos(theta/2.0)) * (r / v * dvdr - 1.0));
         ih2 = I_HII(r, x, y, z, theta, 0.0, mol->nyu0[JU][JL]);
         bt = Tau2Beta(tau);
         sa = SA(theta + dth) - SA(theta);
         sumds    += sa;
         sumbeta  += sa * bt;
         sumibeta += sa * bt * ih2;
      }
   }

/* ----- from neutral gas ----- */
   dth = (PI - theta_h) / (double)THDIV;
   for (theta = theta_h; theta < PI; theta += dth) {
      tau = tau0 / (1.0 + sqr(cos(theta/2.0)) * (r / v * dvdr - 1.0));
      bt = Tau2Beta(tau);
      sa = SA(theta + dth) - SA(theta);
      sumds    += sa;
      sumbeta  += sa * bt;
      sumibeta += sa * bt * BB(T0, mol->nyu0[JU][JL]);
   }

/* ----- return values ----- */
   *beta = sumbeta / sumds;
   *ibeta = sumibeta / sumds;

} /* CalcBeta() */

/* ------------------------------------------------------------- */
void CalcBetaV(r, x, y, z, tk, v, dvdr, JU, JL, mol, beta, ibeta)
/* escape probability for vibrational transition */
/* 93/10/28 new made. */
   double r, x, y, z;      /* position */
   double tk;              /* kinetic temperature */
   double v;               /* radial velocity */
   double dvdr;            /* dv / dr */
   int JU, JL;             /* Upper and lower sstate */
   MOLECULE *mol;          /* molecular data */
   double *beta, *ibeta;   /* escape probabilities */
{
#define THDIV 10     /* number of step of theta */
#define SA(theta) (1.0-cos(theta)) /* solid angle / 4*PI*2 */
   
   double theta, theta_s, theta_h, dth;
   double tau0, tau, bt, sa;
   double ih2;

   double sumds = 0;
   double sumbeta = 0;
   double sumibeta = 0;

   tau0 = cub(C / mol->nyu0[JU][JL]) * mol->g[JU] * mol->a[JU][JL]
         * (mol->ng[JL] - mol->ng[JU]) / (8.0 * PI) * r / v;

/* ----- from central star ----- */
   theta_s = atan(R_Star() / r);
   tau = tau0 / (1.0 + sqr(cos(theta_s/2.0)) * (dvdr - 1.0));

   bt = Tau2Beta(tau);
   sa = SA(theta_s);
   sumds    += sa;
   sumbeta  += sa * bt;
   sumibeta += sa * bt * BB(T_Star(), mol->nyu0[JU][JL]);

/* ----- from HII region ----- */
   if ((theta_h = atan(RO_HII())) > theta_s) {
      dth = (theta_h - theta_s) / (double)THDIV;
      for (theta = theta_s; theta < theta; theta += dth) {

         tau = tau0 / (1.0 + sqr(cos(theta/2.0)) * (r / v * dvdr - 1.0));
         ih2 = I_HII(r, x, y, z, theta, 0.0, mol->nyu0[JU][JL]);
         bt = Tau2Beta(tau);
         sa = SA(theta + dth) - SA(theta);
         sumds    += sa;
         sumbeta  += sa * bt;
         sumibeta += sa * bt * ih2;
      }
   }

/* ----- from neutral gas ----- */
   dth = (PI - theta_h) / (double)THDIV;
   for (theta = theta_h; theta < PI; theta += dth) {
      tau = tau0 / (1.0 + sqr(cos(theta/2.0)) * (r / v * dvdr - 1.0));
      bt = Tau2Beta(tau);
      sa = SA(theta + dth) - SA(theta);
      sumds    += sa;
      sumbeta  += sa * bt;
      sumibeta += sa * bt * BB(T0, mol->nyu0[JU][JL]);
   }

/* ----- return values ----- */
   *beta = sumbeta / sumds;
   *ibeta = sumibeta / sumds;

} /* CalcBetaV() */

/* end of rate.c */
