/* trans.c */
/* new made 93/08/20 I.Yamamura */

#include <stdio.h>
#include <math.h>
#include <malloc.h>

#include "const.h"         /* physical constants */
#include "macro.h"         /* common macro functions */
#include "funcs.h"

#include "mol.h"
#include "rt.h"

#define iv2f(iv)  ((iv + 0.5 - f->rpix) * f->delt + f->rval)
#define f2iv(nyu) (nint((nyu - f->rval) / f->delt + f->rpix))
#define ivcheck(iv) { \
   if (iv < 0)       iv = 0; \
   if (iv > f->n - 1)  iv = f->n - 1; \
}

/* ======================================================================== */
void Trans(y, z, JU, vsys, mol, cx, f, tb, mode)
/* calculate emergency intensity at (y, z) */
   double y, z;      /* cordinate */
   int JU;           /* upper state J number */
   double vsys;      /* systemic velocity */
   MOLECULE *mol;    /* molecular data (only constants are used) */
   LOS *cx;          /* data along line of sight */
   AXIS *f;          /* frequency axis data */
   double *tb;       /* output intensity */
   int mode;         /* if 0, tau is calculated by LVG */
{
   int JL = JU - 1;
   int ix, iv;       /* counter */
   int ivs, ive;
   double te, ne;    /* electron temperature and density */
   double tau, tau0;
   double et;        /* exp(tau) */
   double r;         /* radius */
   double x, dx;
   double v, vx;
   double nyu;
   double cs2;
   double si;        /* source function */
   double fturb, dft;
   double sumphai, dphai;
   static double *phai;
   static double *count, *bb0;
   static int FIRST = 0;

   double nyu0 = mol->nyu0[JU][JL];
   double ryz = sqrt(sqr(y) + sqr(z));
   double nyusys;

   nyusys = Doppler(vsys, nyu0) - nyu0;

   if (FIRST == 0) {
      phai = (double *)calloc(f->n, sizeof(double));
      bb0  = (double *)calloc(f->n, sizeof(double));
      for (iv = 0; iv < f->n; iv++) bb0[iv] = BB(T0, iv2f(iv));
      if (mode == LVG)
         count = (double *)calloc(f->n, sizeof(double));
      FIRST = -1;
   }
   for (iv = 0; iv < f->n; iv++) tb[iv] = bb0[iv];
   if (mode == LVG)
      for (iv = 0; iv < f->n; iv++) count[iv] = 0.0;

   for (ix = 0; ix < cx->n; ix++) {
      x = cx->x[ix];
      if (ix == 0)   dx = fabs(cx->x[1      ] - cx->x[0      ]);
      else if (ix == cx->n - 1)
                     dx = fabs(cx->x[cx->n-1] - cx->x[cx->n-2]);
      else           dx = fabs(cx->x[ix + 1 ] - cx->x[ix - 1 ]) / 2.0;
      r = sqrt(sqr(x) + sqr(ryz));
      if (r == 0.0) r = fabs(dx / 100.0);
      v = cx->v[ix];
      vx = v * (x / r);
      cs2 = sqr(x / r);
      nyu = Doppler(vx, nyu0);

      if (cx->lc[ix] == LINE) {
         si = 1.0 / (cx->ngl[ix] / cx->ngu[ix] - 1.0);
         if (mode == LVG) {
            iv = f2iv(nyu + nyusys);
            if (0 < iv && iv < f->n) {
               tau = cub(C / nyu0) * mol->g[JU] * mol->a[JU][JL]
                  * (cx->ngl[ix] - cx->ngu[ix]) / (8.0 * PI)
                  * r / (v * ((1.0 - cs2) + cs2 * cx->dvdr[ix]));
               et = exp(-tau);
               tb[iv] = tb[iv]*count[iv] + bb0[iv]*et + si*(1.0 - et);
               count[iv] += 1.0;
               tb[iv] /= count[iv];
            }
         } else {
            fturb = (cx->turb[ix] / C) * nyu0;
            ivs = f2iv(nyu + nyusys - 5 * fturb);  ivcheck(ivs);
            ive = f2iv(nyu + nyusys + 5 * fturb);  ivcheck(ive);
            sumphai = 0.0;
            for (iv = ivs; iv <= ive; iv++)
               phai[iv] = 0.0;
            for (dft = - 5*fturb; dft < 5*fturb; dft += fturb / 10.0) {
               iv = (nyu + nyusys + dft - f->rval) / f->delt + f->rpix;
               dphai = exp(-sqr(dft / fturb) / 2.0);
               if ((0 <= iv) && (iv < f->n)) phai[iv] += dphai;
               sumphai += dphai;
            }
            tau0 = sqr(C / nyu0) * mol->g[JU] * mol->a[JU][JL]
               * (cx->ngl[ix] - cx->ngu[ix]) / (8.0 * PI)
               * dx / fabs(f->delt);
            for (iv = ivs; iv <= ive; iv++) {
               tau = tau0 * phai[iv] / sumphai;
               et = exp(-tau);
               tb[iv] = tb[iv] * et + si * (1.0 - et);
            }
         }
      } else if (cx->lc[ix] == CONT) {
         te = cx->ngu[ix];
         ne = cx->ngl[ix];
         if (te > 0.0) {
            tau = 9.77e-3 * sqr(ne) / (te * sqrt(te)) / sqr(nyu)
               * (17.7 + log(te * sqrt(te) / nyu)) * dx;
            et = exp(-tau);
            for (iv = 0; iv < f->n; iv++)       
               tb[iv] = tb[iv] * et + BB(te, nyu) * (1.0 - et);
         }
      } else {
         fprintf(stderr, "Trans : Here is not LINE nor CONT\n");
      }
   }

   for (iv = 0; iv < f->n; iv++)
      tb[iv] -= BB(T0, iv2f(iv));

} /* Trans() */

/* end of trans.c */
